' Program to demonstrate an interrupt-driven input buffer for HRSIN using On Interrupt.


		Include "PROTON_4.INC"


		RCIF	Var	PIR1.5		' Alias RCIF (USART Receive Interrupt Flag)
		OERR	Var	RCSTA.1		' Alias OERR (USART Overrun Error Flag)
		CREN	Var	RCSTA.4		' Alias CREN (USART Continuous Receive Enable)
		LED     Var	PORTD.0 	' Alias LED to PORTD.0

		Buffer_size	Con	16			' Sets the size of the ring buffer
		Buffer		Var	Byte[buffer_size]	' Array variable for holding received characters
		Index_in	Var	Byte		' Pointer - next empty location in buffer
		Index_out	Var	Byte		' Pointer - location of oldest character in buffer
		Bufchar		Var	Byte		' Stores the character retrieved from the buffer
		I			Var	Byte		' loop counter 
		Col			Var	Byte		' Stores location on LCD for text wrapping
		Errflag		Var	Byte		' Holds error flags

' Initialize variables

		Index_in = 0
		Index_out = 0
		I = 0
		Col = 1


		Cls								' Clear LCD
		INTCON = %11000000				' Enable interrupts
		On Interrupt Goto Serialin		' Declare interrupt handler routine
		PIE1.5 = 1						' Enable interrupt on USART

' Main program starts here - blink an LED at 1Hz

loop:   High LED        				' Turn on LED connected to PORTD.0
        For I = 0 to 250				' Delay for .5 seconds (250*2mS)
	    	Pause 2   					' Use a short pause within a loop
		Next							' instead of one long pause
        Low LED     					' Turn off LED connected to PORTD.0
        For I = 0 to 250				' Delay for .5 seconds (250*2mS)
	        Pause 2   					' Use a short pause within a loop
		Next							' instead of one long pause

display:								' dump the buffer to the LCD
		If Errflag  > 0 Then Error1		' Handle error if needed
		If Index_in = Index_out Then Loop	' loop if nothing in buffer

		Gosub Getbuf					' Get a character from buffer	
		Lcdout Bufchar					' Send the character to LCD
		
		Col = Col + 1					' Increment LCD location
		If Col > 20 Then				' Check for end of line
			Col = 1						' Reset LCD location
			Lcdout at 2,1,Rep " "\20 , 254,2	' Clear line-2 of LCD and Tell LCD to return home
		Endif

		Goto Display			' Check for more characters in buffer



' Subroutines

		Disable							' Don't check for interrupts in this section

getbuf:									' move the next character in buffer to bufchar
	Index_out = (Index_out + 1)			' Increment index_out pointer (0 to 63)
	If Index_out > (Buffer_size - 1) Then Index_out = 0	' Reset pointer if outside of buffer
	Bufchar = Buffer[Index_out]			' Read buffer location
Return


Error1:									' Display error message if buffer has overrun
	If Errflag.1  = 1 Then				' Determine the error
		Lcdout at 2,1,"Buffer Overrun"	' Display buffer error on line-2
	Else
		Lcdout at 2,1,"USART Overrun"	' Display usart error on line-2
	EndIF
	
	Lcdout $FE,2						' Send the LCD cursor back to line-1 home
	For I = 2 to Col					' Loop for each column beyond 1
		Lcdout $FE,$14					' Move the cursor right to the right column
	Next
	
	Errflag = 0							' Reset the error flag
	CREN = 0							' Disable continuous receive to clear overrun flag
	CREN = 1							' Enable continuous receive

	Goto Display						' Carry on
	
	
' Interrupt handler

Serialin:								' Buffer the character received
	If OERR  = 1 Then Usart_error			' Check for USART errors
	Index_in = (Index_in + 1)			' Increment index_in pointer (0 to 63)
	If Index_in > (Buffer_size - 1) Then Index_in = 0	'Reset pointer if outside of buffer
	If Index_in = Index_out Then buffer_error	' Check for buffer overrun
	Hrsin Buffer[Index_in]						' Read USART and store character to next empty location
	If RCIF = 1 Then Serialin						' Check for another character while we're here
	
	Resume										' Return to program

Buffer_error:
	Errflag.1 = 1								' Set the error flag for software
' Move pointer back to avoid corrupting the buffer. MIN insures that it ends up within the buffer.	
	Index_in = (Index_in - 1) Min (Buffer_size - 1)	
	Hrsin Buffer[Index_in]						' Overwrite the last character stored (resets the interrupt flag)
Usart_error:
	Errflag.0 = 1								' Set the error flag for hardware
	
	Resume										' Return to program
				
