' Demonstrate wake on interrupt.

' You should DISABLE THE WATCHDOG TIMER when programming the
' PICmicro device.

' The program outputs the condition of the switches on the
' LEDs, then immediately goes into power-down mode.  When
' the condition on any switch changes, the RB port change
' interrupt occurs, waking the part.
' Since the global interrupt enable bit is not set, no 
' jump to the interrupt vector occurs.  Program execution 
' is resumed, the LEDs are updated, and the part goes back
' to sleep.
' To further reduce the power consumption, all unused
' hardware peripherals should be disabled.  The PORTB
' pullups should be replaced with external resistors, and
' the internal pullups disabled.

		Include "PROTON_4.INC"
        
' Define the pins that are connected to pushbuttons.
' The switches must be connected to PORTB, pins 4,5,6,
' or 7 in order to use the RB port change interrupt.
		SW1	Var	PORTB.4
		SW2	Var PORTB.5
		SW3	Var	PORTB.6

' Define the pins that are connected to LEDs
		LED1	Var	PORTB.0
		LED2	Var	PORTB.1
		LED3	Var	PORTB.2

		INTCON.3 = 1			' Enable the RB port change interrupt
		OPTION_REG = $7F		' Enable PORTB pull-ups
		TRISB = %11111000		' Set PORTB.0-2 (LEDs) to output, 3-7 to input

' MAIN PROGRAM BEGINS HERE
main:	

		PORTB = 0				' Turn off all LEDs

' Check any button pressed to toggle on LED
		If sw1 = 0 Then LED1 = 1
		If sw2 = 0 Then LED2 = 1
		If sw3 = 0 Then LED3 = 1
	
		INTCON.0 = 0			' Clear the RB port change flag bit
	
		Nap 7					' Go to sleep.  When the watchdog is
								' disabled, NAP won't wake up until
								' an interrupt occurs.

		Goto main				' Do it again upon waking

